---------------------------------------------------------------
-- Table-Valued Parameters
---------------------------------------------------------------

USE AdventureWorks2012
GO

-- Listing 1
CREATE TYPE CustomerUdt AS TABLE
  (Id int,
   CustomerName nvarchar(50),
   PostalCode nvarchar(50))

/* TVP Order Entry */

USE MyDB
GO

CREATE TABLE [Order](
 OrderId int NOT NULL,
 CustomerId int NOT NULL,
 OrderedAt datetime NOT NULL,
  CONSTRAINT PK_Order PRIMARY KEY CLUSTERED (OrderId ASC))

GO

CREATE TABLE [OrderDetail](
 OrderId int NOT NULL,
 LineNumber int NOT NULL,
 ProductId int NOT NULL,
 Quantity int NOT NULL,
 Price money NOT NULL,
  CONSTRAINT PK_OrderDetail PRIMARY KEY CLUSTERED (OrderId ASC, LineNumber ASC))

GO

CREATE TYPE OrderUdt AS TABLE(
 OrderId int,
 CustomerId int,
 OrderedAt datetime2(0))

CREATE TYPE OrderDetailUdt AS TABLE(
 OrderId int,
 LineNumber int,
 ProductId int,
 Quantity int,
 Price money)

GO 

-- Pass TVP from remote client to to stored proc
CREATE PROCEDURE uspInsertNewOrder(
 @OrderHeader AS OrderUdt READONLY,
 @OrderDetails AS OrderDetailUdt READONLY)
AS 
BEGIN

    -- Pass TVP from one stored proc to another
    EXEC uspInspectDetails @OrderDetails

    -- Insert order header row from TVP
    INSERT INTO [Order]
     SELECT * FROM @OrderHeader

    -- Bulk insert order detail rows from TVP
    INSERT INTO [OrderDetail]
     SELECT * FROM @OrderDetails

END
GO

CREATE PROCEDURE uspInspectDetails(
 @OrderDetails AS OrderDetailUdt READONLY)
AS
BEGIN

    SELECT
     'Inspected' AS Inspected,
     dbo.udfGetDetailCount(@OrderDetails) AS DetailCount, -- Pass TVP from stored proc to UDF
     *
    FROM
     @OrderDetails

END
GO

CREATE FUNCTION udfGetDetailCount(
 @OrderDetails AS OrderDetailUdt READONLY)
 RETURNS int
AS
BEGIN
    DECLARE @result AS int
    SELECT @result = COUNT(*) FROM @OrderDetails
    RETURN @result
END
GO

-- Pilot
DECLARE @OrderHeader AS OrderUdt
DECLARE @OrderDetails AS OrderDetailUdt

INSERT INTO @OrderHeader VALUES(1, 1, '2/9/2008')
INSERT INTO @OrderDetails VALUES
 (1, 1, 329, 1, 24.95),
 (1, 2, 247, 2, 15.95),
 (1, 3, 193, 4, 9.50)

EXECUTE uspInsertNewOrder @OrderHeader, @OrderDetails

SELECT * FROM [Order]
SELECT * FROM [OrderDetail]

-- Cleanup
DROP FUNCTION udfGetDetailCount
DROP PROCEDURE uspInspectDetails
DROP PROCEDURE uspInsertNewOrder	
DROP TYPE OrderUdt
DROP TYPE OrderDetailUdt
DROP TABLE [Order]
DROP TABLE [OrderDetail]
GO

/* TVP Order Entry - for Business Collection demo */

USE MyDB
GO

CREATE TABLE [Order](
 OrderId int NOT NULL,
 CustomerId int NOT NULL,
 OrderedAt date NOT NULL,
 CreatedAt datetime2(0) NOT NULL,
  CONSTRAINT PK_Order PRIMARY KEY CLUSTERED (OrderId ASC))

GO

CREATE TABLE [OrderDetail](
 OrderId int NOT NULL,
 LineNumber int NOT NULL,
 ProductId int NOT NULL,
 Quantity int NOT NULL,
 Price money NOT NULL,
 CreatedAt datetime2(0) NOT NULL,
  CONSTRAINT PK_OrderDetail PRIMARY KEY CLUSTERED (OrderId ASC, LineNumber ASC))

GO

CREATE TYPE OrderUdt AS TABLE(
 OrderId int,
 CustomerId int,
 OrderedAt date)

CREATE TYPE OrderDetailUdt AS TABLE(
 OrderId int,
 LineNumber int,
 ProductId int,
 Quantity int,
 Price money)

GO 

-- Pass TVP from remote client to to stored proc
CREATE PROCEDURE uspInsertOrders(
 @OrderHeaders AS OrderUdt READONLY,
 @OrderDetails AS OrderDetailUdt READONLY)
AS 
BEGIN

    -- Insert order header rows from TVP
    INSERT INTO [Order]
     SELECT *, SYSDATETIME() FROM @OrderHeaders

    -- Bulk insert order detail rows from TVP
    INSERT INTO [OrderDetail]
     SELECT *, SYSDATETIME() FROM @OrderDetails

END
GO

-- Bulk load using C# collection with custom iterator

SELECT * FROM [Order]
SELECT * FROM [OrderDetail]

-- Cleanup
DROP PROCEDURE uspInsertOrders
DROP TYPE OrderUdt
DROP TYPE OrderDetailUdt
DROP TABLE [OrderDetail]
DROP TABLE [Order]
GO

/* TVP Order Entry - for EF demo */

CREATE TABLE [Order](
 OrderId int IDENTITY NOT NULL PRIMARY KEY,
 CustomerId int NOT NULL,
 OrderedAt date NOT NULL,
 CreatedAt datetime2(0) NOT NULL)

CREATE TABLE [OrderDetail](
 OrderDetailId int IDENTITY NOT NULL PRIMARY KEY,
 OrderId int NOT NULL,
 ProductId int NOT NULL,
 Quantity int NOT NULL,
 Price money NOT NULL,
 CreatedAt datetime2(0) NOT NULL)

ALTER TABLE [OrderDetail] ADD CONSTRAINT FK_OrderDetail_Order
 FOREIGN KEY(OrderId) REFERENCES [Order](OrderId)

CREATE TYPE OrderUdt AS TABLE(
 CustomerId int,
 OrderedAt date)

CREATE TYPE OrderDetailUdt AS TABLE(
 ProductId int,
 Quantity int,
 Price money)
GO 

CREATE PROCEDURE uspInsertOrder(
 @OrderHeader AS OrderUdt READONLY,
 @OrderDetails AS OrderDetailUdt READONLY) AS 
BEGIN

  IF ((SELECT COUNT(*) FROM @OrderHeader) <> 1)
   THROW 50000, 'Exactly one row is expected by the @OrderHeader TVP', 1

  BEGIN TRANSACTION

    INSERT INTO [Order](CustomerId, OrderedAt, CreatedAt)
     OUTPUT inserted.OrderId, inserted.CreatedAt
     SELECT CustomerId, OrderedAt, SYSDATETIME() FROM @OrderHeader

    DECLARE @OrderId int = SCOPE_IDENTITY()

    INSERT INTO [OrderDetail](OrderId, ProductId, Quantity, Price, CreatedAt)
     OUTPUT inserted.OrderDetailId, inserted.CreatedAt
     SELECT @OrderId, ProductId, Quantity, Price, SYSDATETIME() FROM @OrderDetails

  COMMIT TRANSACTION
END
GO

-- Pilot
DECLARE @OrderHeader AS OrderUdt
DECLARE @OrderDetails AS OrderDetailUdt

INSERT INTO @OrderHeader VALUES(1, '2/9/2008')
INSERT INTO @OrderDetails VALUES
 (329, 1, 24.95),
 (247, 2, 15.95),
 (193, 4, 9.50)

EXECUTE uspInsertOrder @OrderHeader, @OrderDetails

SELECT * FROM [Order]
SELECT * FROM [OrderDetail]

-- Cleanup
DROP PROCEDURE uspInsertOrder
DROP TYPE OrderUdt
DROP TYPE OrderDetailUdt
DROP TABLE [OrderDetail]
DROP TABLE [Order]
GO

/* Bulk Insert */

USE AdventureWorks2012
GO

CREATE TYPE LocationUdt AS TABLE 
(
  LocationName varchar(50),
  CostRate int
)
GO

CREATE PROCEDURE uspInsertProductionLocation
 (@TVP LocationUdt READONLY)

    AS 

     INSERT INTO [Production].[Location]
        ([Name],
         [CostRate],
         [Availability],
         [ModifiedDate])
      SELECT *, 0, GETDATE() FROM @TVP

GO

-- Pilot
DECLARE @LocationTvp AS LocationUdt

INSERT INTO @LocationTvp VALUES('UK', 122.4)
INSERT INTO @LocationTvp VALUES('Paris', 359.73)

EXEC uspInsertProductionLocation @LocationTvp
GO

SELECT * FROM Production.Location

-- Bulk Select + Insert
DECLARE @LocationTVP AS LocationUdt

INSERT INTO @LocationTVP
 SELECT [Name], 0.00 FROM [Person].[StateProvince]

EXEC uspInsertProductionLocation @LocationTVP

SELECT * FROM Production.Location


-- Cleanup
DROP PROCEDURE uspInsertProductionLocation
DROP TYPE LocationUdt


/* Bulk Update */

USE MyDB
GO

CREATE TABLE Category
 (Id   int PRIMARY KEY,
  Name nvarchar(max),
  CreatedAt	datetime2(0) DEFAULT SYSDATETIME())

-- Initialize with a few categories
INSERT INTO Category(Id, Name) VALUES(1, 'Housewares')
INSERT INTO Category(Id, Name) VALUES(2, 'Maternity')
INSERT INTO Category(Id, Name) VALUES(3, 'Mens Apparel')
INSERT INTO Category(Id, Name) VALUES(4, 'Womens Apparel')
INSERT INTO Category(Id, Name) VALUES(5, 'Bath')
INSERT INTO Category(Id, Name) VALUES(6, 'Automotive')

-- View the list of categories
SELECT * FROM Category

-- Will be used by uspUpdateCategories to pass in a set of category updates
CREATE TYPE EditedCategoriesUdt AS TABLE
 (Id int PRIMARY KEY,
  Name nvarchar(max))
GO

-- Receive multiple rows for the change set and update the Category table
CREATE PROCEDURE uspUpdateCategories(@EditedCategoriesTVP AS EditedCategoriesUdt
 READONLY)
AS
 BEGIN

  -- Update names in the Category table by joining on the TVP by ID
  UPDATE Category
   SET Category.Name = ec.Name
   FROM Category INNER JOIN @EditedCategoriesTVP AS ec ON Category.Id = ec.Id

 END

-- Load up a few changes into a new TVP instance (to categories 1 and 5)
DECLARE @Edits AS EditedCategoriesUdt
INSERT INTO @Edits VALUES(1, 'Gifts & Housewares')
INSERT INTO @Edits VALUES(5, 'Bath & Kitchen')

-- Call the stored procedure
EXECUTE uspUpdateCategories @Edits

-- View the updated names for categories 1 and 5 in the Category table
SELECT * FROM Category

-- Cleanup
DROP PROCEDURE uspUpdateCategories
DROP TYPE EditedCategoriesUdt
DROP TABLE Category

