/*
****************************************
-- XQuery
****************************************
*/

-- Simple xml.query
DECLARE @Books xml = '
<catalog> 
  <book category="ITPro"> 
    <title>Windows Step By Step</title> 
    <author>Jeff Hay</author> 
    <price>49.99</price> 
  </book> 
  <book category="Developer"> 
    <title>Learning ADO .NET</title> 
    <author>Holly Holt</author> 
    <price>39.93</price> 
  </book> 
  <book category="ITPro"> 
    <title>Administering IIS</title>  
    <author>Jed Brown</author>  
    <price>59.99</price>  
  </book> 
</catalog>'

SELECT @Books.query('
  <ITProBooks>
    {
      for $b in /catalog/book 
      where $b/@category="ITPro" 
      order by $b/author[1] descending  
      return ($b)
    }
  </ITProBooks>')

USE master
GO

IF EXISTS(SELECT name FROM sys.databases WHERE name = 'SampleDB')
 DROP DATABASE SampleDB 
GO

CREATE DATABASE SampleDB
GO

USE SampleDB
GO

-- Create an XML schema definition for speaker classes data
CREATE XML SCHEMA COLLECTION ClassesXSD AS '
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="class">
    <xs:complexType>
      <xs:attribute name="name" type="xs:string" use="required" />
    </xs:complexType>
  </xs:element>
  <xs:element name="classes">
    <xs:complexType>
      <xs:sequence>
        <xs:element ref="class" minOccurs="1" maxOccurs="unbounded" />
      </xs:sequence>
      <xs:attribute name="speakerBio" type="xs:string" use="required" />
    </xs:complexType>
  </xs:element>
</xs:schema>'
GO

-- Create table of speakers with XML column for their classes
CREATE TABLE Speaker(
 SpeakerId int IDENTITY PRIMARY KEY,
 SpeakerName varchar(50),
 Country varchar(25),
 ClassesXML xml(ClassesXSD) NOT NULL)

-- Create primary XML index
CREATE PRIMARY XML INDEX ix_speakers
	ON Speaker(ClassesXML)

-- Create secondary structural (path) XML index
CREATE XML INDEX ix_speakers_path ON Speaker(ClassesXML)
 USING XML INDEX ix_speakers FOR PATH 

-- Insert a few speakers, including an invalid one
INSERT INTO Speaker VALUES('Jeff Hay', 'USA', '
  <classes speakerBio="Jeff has solid security experience from years of hacking">
    <class name="Writing Secure Code for ASP .NET" />
    <class name="Using XQuery to Manipulate XML Data in SQL Server 2012" />
    <class name="SQL Server and Oracle Working Together" />
    <class name="Protecting against SQL Injection Attacks" />
  </classes>')

INSERT INTO Speaker VALUES('Holly Holt', 'Canada', '
  <classes speakerBio="Holly is a Canadian-born database professional">
    <class name="SQL Server Profiler" />
    <class name="Advanced SQL Querying Techniques" />
    <class name="SQL Server and Oracle Working Together" />
  </classes>')

INSERT INTO Speaker VALUES('Jed Brown', 'USA', '
  <classes speakerBio="Jed specializes in client development">
    <class name="Smart Client Stuff" />
    <class name="More Smart Client Stuff" />
  </classes>')

INSERT INTO Speaker VALUES('Bad Speaker', 'France', '
  <CLASSES SPEAKERBIO="Jean has case-sensitivity issues">
	  <class name="SQL Server Index" />
	  <class name="SQL Precon" />
  </CLASSES>')

-- Simple xml.exist
DECLARE @SomeData xml = '
<classes>
	<class name="SQL Server Index"/>
	<class name="SQL Precon"/>
</classes>'

SELECT
 @SomeData.exist('/classes') AS HasClasses,
 @SomeData.exist('/dogs') AS HasDogs

-- xml.exist testing for substring in name attribute
SELECT * FROM Speaker
 WHERE
  ClassesXML.exist('/classes/class/@name[contains(., "SQL Server")]') = 1

-- xml.exist combined with ordinary column-based filtering logic
SELECT * FROM Speaker
 WHERE
  ClassesXML.exist('/classes/class/@name[contains(., "SQL Server")]') = 1
  AND Country = 'USA'

-- xml.value
SELECT
  SpeakerName,
  Country,
  ClassesXML.value('/classes[1]/@speakerBio','varchar(max)') AS SpeakerBio,
  ClassesXML.value('count(/classes/class)', 'int') AS SessionCount
 FROM
  Speaker
 ORDER BY
  ClassesXML.value('count(/classes/class)', 'int')

-- xml.query with FLWOR and sql:column for XML transformations
SELECT
  SpeakerId,
  ClassesXML.query('
    let $c := count(/classes/class)
    let $b := data(/classes[1]/@speakerBio)
    return
      <SpeakerInfo>
        <Name>{sql:column("SpeakerName")}</Name>
        <Country>{sql:column("Country")}</Country>
        <Bio>{$b}</Bio>
        <Sessions count="{$c}">
          {
            for $s in /classes/class
            let $n := data($s/@name)
            order by $n
            return
              <Session>{$n}</Session>
          }
        </Sessions>
      </SpeakerInfo>
	  ') AS SpeakerInfo
 FROM
  Speaker

-- xml.query with sql:variable for parameterized transformations
DECLARE @Category varchar(max) = 'SQL Server'

SELECT
  SpeakerName,
  Country,
  ClassesXML.query('
    <classes
      category="{sql:variable("@Category")}"
      speakerBio="{data(/classes[1]/@speakerBio)}">
      {
        for $c in /classes/class
        where $c/@name[contains(., sql:variable("@Category"))]
        return $c
      }
    </classes>') AS ClassesXML
 FROM
  Speaker
 WHERE
  ClassesXML.exist
   ('/classes/class/@name[contains(., sql:variable("@Category"))]') = 1


SELECT * from Speaker

-- xml.xquery with CONVERT to combine child elements
SELECT
  SpeakerName,
  Country,
  CONVERT(varchar(max), ClassesXML.query('
    for $s in /classes/class
    let $n := data($s/@name)
    let $p := concat($n, "|")
    return $p')) AS SessionList
 FROM
  Speaker

/*
****************************************
-- XML DML
****************************************
*/

-- xml.modify insert
UPDATE Speaker
 SET ClassesXML.modify('
  insert 
    <class name="Ranking and Windowing Functions in SQL Server" />
  into
    /classes[1]')
 WHERE SpeakerId = 1

SELECT * from Speaker

-- xml.modify delete
UPDATE Speaker
 SET ClassesXML.modify('delete /classes/class[4]')
 WHERE SpeakerId = 1

SELECT * from Speaker

-- xml.modify replace
UPDATE Speaker
 SET ClassesXML.modify('
  replace value of /classes[1]/class[3]/@name[1]
	with "Getting SQL Server and Oracle to Work Together"')
 WHERE SpeakerId = 1

SELECT * from Speaker


-- Cleanup

DROP TABLE Speaker
DROP XML SCHEMA COLLECTION ClassesXSD
