﻿USE master
GO
IF EXISTS(SELECT name FROM sys.databases WHERE name = 'SampleDb')
 DROP DATABASE SampleDb
GO
CREATE DATABASE SampleDb
GO
USE SampleDb
GO

CREATE TABLE Customer(
  CustomerId bigint IDENTITY(1,1) NOT NULL,
  FirstName varchar(50) NOT NULL,
  LastName varchar(50) NOT NULL,
  Balance money NOT NULL DEFAULT 0,
  CreatedAt datetime2(7) NOT NULL DEFAULT SYSDATETIME(),
  UpdatedAt datetime2(7) NOT NULL DEFAULT SYSDATETIME(),
  CONSTRAINT PK_Customer PRIMARY KEY CLUSTERED (CustomerId ASC))

CREATE TABLE OrderHeader(
  OrderHeaderId bigint IDENTITY(1,1) NOT NULL,
  CustomerId bigint NOT NULL,
  ShipVia varchar(20) NOT NULL,
  OrderStatus varchar(20) NULL,
  Notes varchar(max) NULL,
  CreatedAt datetime2(7) NOT NULL DEFAULT SYSDATETIME(),
  UpdatedAt datetime2(7) NOT NULL DEFAULT SYSDATETIME(),
  CONSTRAINT PK_OrderHeader PRIMARY KEY CLUSTERED (OrderHeaderId ASC))

ALTER TABLE OrderHeader WITH CHECK ADD CONSTRAINT FK_OrderHeader_Customer
 FOREIGN KEY(CustomerId) REFERENCES Customer(CustomerId)

CREATE TABLE Employee(
  EmployeeId bigint IDENTITY(1,1) NOT NULL,
  FirstName varchar(50) NOT NULL,
  LastName varchar(50) NOT NULL,
  Salary decimal(18, 9) NOT NULL,
  CreatedAt datetime2(7) NOT NULL,
  UpdatedAt datetime2(7) NOT NULL,
  CONSTRAINT PK_Employee PRIMARY KEY CLUSTERED (EmployeeId ASC))

CREATE TABLE CustomerEmployee(
  CustomerId bigint NOT NULL,
  EmployeeId bigint NOT NULL,
  CONSTRAINT
   PK_CustomerEmployee PRIMARY KEY CLUSTERED (CustomerId ASC, EmployeeId ASC))
  
ALTER TABLE CustomerEmployee
 WITH CHECK ADD CONSTRAINT FK_CustomerEmployee_Customer
 FOREIGN KEY(CustomerId) REFERENCES Customer(CustomerId)

ALTER TABLE CustomerEmployee
 WITH CHECK ADD CONSTRAINT FK_CustomerEmployee_Employee
 FOREIGN KEY(EmployeeId) REFERENCES Employee(EmployeeId)

GO

CREATE PROCEDURE SelectCustomer(@CustomerId bigint)
AS
  SELECT    CustomerId, FirstName, LastName, Balance, CreatedAt, UpdatedAt
   FROM     Customer
   WHERE    CustomerId = @CustomerId
GO

CREATE PROCEDURE SelectCustomers
AS
  SELECT    CustomerId, FirstName, LastName, Balance, CreatedAt, UpdatedAt
   FROM     Customer
   ORDER BY CustomerId
GO

CREATE PROCEDURE InsertCustomer(
  @FirstName varchar(20),
  @LastName varchar(20),
  @Balance money = 0)
AS
  DECLARE @CreatedAt datetime2 = SYSDATETIME()

  INSERT INTO Customer (FirstName, LastName, Balance, CreatedAt, UpdatedAt)
   VALUES (@FirstName, @LastName, @Balance, @CreatedAt, @CreatedAt)

  SELECT
    CustomerId = CAST(SCOPE_IDENTITY() AS bigint),
    CreatedAt = @CreatedAt,
    UpdatedAt = @CreatedAt
GO

CREATE PROCEDURE UpdateCustomer(
  @CustomerId bigint,
  @FirstName varchar(20),
  @LastName varchar(20),
  @Balance money,
  @OriginalUpdatedAt datetime2)
AS
  DECLARE @NewUpdatedAt datetime2 = SYSDATETIME()

  UPDATE Customer
   SET
    FirstName = @FirstName,
    LastName = @LastName,
    Balance = @Balance,
    UpdatedAt = @NewUpdatedAt
   WHERE
    CustomerId = @CustomerId AND UpdatedAt = @OriginalUpdatedAt

  IF @@ROWCOUNT = 0
    THROW 50000, 'The customer does not exist, or has been updated/deleted', 1;
  ELSE
    SELECT UpdatedAt = @NewUpdatedAt
GO

CREATE PROCEDURE DeleteCustomer(@CustomerId bigint)
AS
  DELETE  Customer
   WHERE  CustomerId = @CustomerId
GO

CREATE PROCEDURE GetCustomerBalance(
  @CustomerId bigint,
  @Balance money OUTPUT)
AS
  SELECT  @Balance = Balance
   FROM   Customer
   WHERE  CustomerId = @CustomerId
GO
